# -*- coding: utf-8 -*-
"""Definition of places app urls."""
from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib.auth.decorators import login_required

from places.views import (PlaceAddView, PlaceDetailView, PlaceListView,
                          UserPlaceListView)

urlpatterns = [
    url(r'^$', PlaceListView.as_view(), name='places-index'),
    url(r'^add/$', login_required(PlaceAddView.as_view()), name='places-new'),
    url(r'^filter/(?P<category_slug>[-\w]+)/$', PlaceListView.as_view(), name='places-category'),
    url(r'^by/(?P<username>[-\w]+)/$', UserPlaceListView.as_view(), name='places-user-list'),
    url(r'^(?P<slug>[-\w]+)/$', PlaceDetailView.as_view(), name='places-detail'),
]



# -*- coding: utf-8 -*-
""" Utils functions for places app """

import base64
import hashlib
import hmac
try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


def sign_googleapi_url(input_url, secret):
    """ Sign a request URL with a URL signing secret.
    Usage:
    from places.utils import sign_googleapi_url
    signed_url = sign_googleapi_url(input_url=my_url, secret=SECRET)
    Args:
    input_url - The URL to sign
    secret    - Your URL signing secret
    Returns:
    The signed request URL
    """
    input_url = input_url.encode('utf-8')
    url = urlparse(input_url)

    # We only need to sign the path+query part of the string
    url_to_sign = url.path + b"?" + url.query

    # Decode the private key into its binary format
    # We need to decode the URL-encoded private key
    decoded_key = base64.urlsafe_b64decode(secret)

    # Create a signature using the private key and the URL-encoded
    # string using HMAC SHA1. This signature will be binary.
    signature = hmac.new(decoded_key, url_to_sign, hashlib.sha1)

    # Encode the binary signature into base64 for use within a URL
    encoded_signature = base64.urlsafe_b64encode(signature.digest())

    original_url = url.scheme + b"://" + url.netloc + url.path + b"?" + url.query

    # Return signed URL
    return original_url + b"&signature=" + encoded_signature



# -*- coding: utf-8 -*-
"""Managers for the app models."""

from __future__ import unicode_literals

from django.conf import settings
from django.contrib.gis.db import models
from django.db.models import Q


class PlaceManager(models.GeoManager):
    def for_site(self, site_id=None):
        if site_id is None:
            site_id = settings.SITE_ID
        return self.filter(site__id=site_id)

    def public(self, site_id=None):
        """
        Return the public places.

        By default for the current website, or for the site whose ID is passed
        as *site_id* parameter.
        """
        queryset = self.for_site(site_id)
        queryset = queryset.filter(is_public=True)
        return queryset

    def for_user(self, user, site_id=None):
        filters = Q(is_public=True) | Q(is_public=False, user=user)
        queryset = self.for_site(site_id)
        queryset = queryset.filter(filters)
        return queryset

    def user_public_places(self, user, site_id=None):
        return self.for_site(site_id).filter(is_public=True, user=user)



# -*- coding: utf-8 -*-
"""Views for places app."""
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import slugify
from django.views.generic import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView

from places.forms import AddPlaceForm
from places.models import Category, Place


class PlaceListView(ListView):
    """List all the public places."""

    def get_queryset(self):
        qs = Place.objects.public()
        if 'category_slug' in self.kwargs:
            qs = qs.filter(category=get_object_or_404(Category, slug=self.kwargs['category_slug']))
        return qs

    def get_context_data(self, **kwargs):
        context = super(PlaceListView, self).get_context_data(**kwargs)
        context['current_category'] = None
        if 'category_slug' in self.kwargs:
            context['current_category'] = get_object_or_404(Category,
                                                            slug=self.kwargs['category_slug'])
        return context

class UserPlaceListView(ListView):
    """List the places for a given user."""

    def get_queryset(self):
        username = self.kwargs.get('username')
        user = get_object_or_404(User, username=username)
        if user == self.request.user:
            queryset = Place.objects.for_user(user=user)
        else:
            queryset = Place.objects.user_public_places(user=user)
        return queryset

    def get_context_data(self, **kwargs):
        context = super(UserPlaceListView, self).get_context_data(**kwargs)
        context['center'] = self.get_queryset().collect().centroid
        return context


class PlaceDetailView(DetailView):
    slug_field = 'slug'

    def get_queryset(self):
        if self.request.user.is_authenticated():
            queryset = Place.objects.for_user(self.request.user)
        else:
            queryset = Place.objects.public()
        return queryset

    def get_context_data(self, **kwargs):
        context = super(PlaceDetailView, self).get_context_data(**kwargs)
        nearby_items = getattr(settings, 'PLACES_RELATED_COUNT', 5)
        try:
            # TODO: put as place method
            if self.request.user.is_authenticated():
                nearby = Place.objects.for_user(self.request.user).distance(self.object.position).exclude(id=self.object.id)[:nearby_items]
            else:
                nearby = Place.objects.public().distance(self.object.position).exclude(id=self.object.id)[:nearby_items]
        except ValueError:
            nearby = Place.objects.public().order_by('?')[:nearby_items]
        context['nearby'] = nearby
        return context


class PlaceAddView(CreateView):
    template_name = 'places/place_form.html'

    def get_form_class(self):
        return AddPlaceForm

    def form_valid(self, form):
        form.instance.user = self.request.user
        form.instance.slug = slugify(form.instance.name)
        return super(PlaceAddView, self).form_valid(form)



# -*- coding: utf-8 -*-
"""Places app forms."""
from django import forms
from django.template.defaultfilters import slugify

from places.models import Place


class AddPlaceForm(forms.ModelForm):
    class Meta:
        model = Place
        fields = ('name', 'position', 'city', 'is_public', 'description', 'site')
        widgets = {
            'site': forms.HiddenInput(),
        }

    def clean(self):
        if 'name' in self.cleaned_data:
            slug = slugify(self.cleaned_data['name'])
            try:
                Place.objects.get(site=self.cleaned_data['site'], slug=slug)
                raise forms.ValidationError("slug already exist.")
            except Place.DoesNotExist:
                pass
        return self.cleaned_data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.db import models
from django.contrib.sites.models import Site
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from places.managers import PlaceManager


@python_2_unicode_compatible
class Category(models.Model):
    # site = models.ForeignKey(Site, default=settings.SITE_ID)
    name = models.CharField(_("Name"), max_length=200)
    slug = models.SlugField(_("Slug"))
    # private_icon
    # public_icon

    def __str__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('places-category', [self.slug])


@python_2_unicode_compatible
class Place(models.Model):
    site = models.ForeignKey(Site, default=settings.SITE_ID)
    user = models.ForeignKey(User)
    category = models.ForeignKey(Category)
    name = models.CharField(_("Name"), max_length=200)
    slug = models.SlugField(_("Slug"))

    longitude = models.FloatField(_("Longitude"), blank=True, editable=False)
    latitude = models.FloatField(_("Latitude"), blank=True, editable=False)
    position = models.PointField(_("Position"), srid=4326, blank=True, null=True)
    is_public = models.BooleanField(_("Is public?"), db_index=True, default=False)

    street = models.CharField(_("Street"), blank=True, max_length=250)
    postal_code = models.CharField(_("Postal code"), blank=True, max_length=25)
    city = models.CharField(_("City"), blank=True, max_length=100)

    description = models.TextField(_("Description"), blank=True)

    added_on = models.DateTimeField(default=datetime.datetime.now, editable=False)
    updated_on = models.DateTimeField(editable=False)

    objects = PlaceManager()

    # place / people / club / boutique

    class Meta:
        unique_together = (
            ('site', 'slug')
        )
        verbose_name = _('place')
        verbose_name_plural = _('places')

    @models.permalink
    def get_absolute_url(self):
        return ('places-detail', [self.slug])

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        """Updating latitude/longitude when position is updated."""
        if self.position:
            self.latitude = self.position.x
            self.longitude = self.position.y
        self.updated_on = datetime.datetime.now()
        super(Place, self).save(*args, **kwargs)
# 
# 
# class PlaceLink(models.Model):
#     WEBSITE = 'website'
#     FACEBOOK = 'facebook'
#     TWITTER = 'twitter'
#     INSTAGRAM = 'instagram'
# 
#     LINK_TYPES = (
#         (WEBSITE, 'Website'),
#         (FACEBOOK, 'Facebook'),
#         (TWITTER, 'Twitter'),
#         (INSTAGRAM, 'Instagram'),
#     )
#     place = models.ForeignKey(Place)
#     url = models.URLField()
#     link_type = models.CharField(choices=LINK_TYPES, default=WEBSITE)



# -*- coding: utf-8 -*-
"""Places admin."""
from __future__ import unicode_literals

from django.contrib.gis import admin
from django.utils.translation import ugettext_lazy as _
from places.models import Category, Place


class PlaceAdmin(admin.OSMGeoAdmin):
    list_display = ('name', 'position', 'is_public', 'category', 'user', 'city', 'added_on', 'updated_on')
    list_filter = ('site', 'category', 'is_public', 'added_on')
    date_hierarchy = ('added_on')
    prepopulated_fields = {"slug": ("name",)}
    search_fields = ('name', 'description',)
    actions = ['mark_as_public', 'mark_as_private']

    def mark_as_public(self, request, queryset):
        queryset.update(is_public=True)
    mark_as_public.short_description = _("Mark as public")

    def mark_as_private(self, request, queryset):
        queryset.update(is_public=False)
    mark_as_private.short_description = _("Mark as private")


class CategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug')
    prepopulated_fields = {"slug": ("name",)}

admin.site.register(Place, PlaceAdmin)
admin.site.register(Category, CategoryAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-30 21:03
from __future__ import unicode_literals

import datetime
from django.conf import settings
import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
            ],
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('longitude', models.FloatField(blank=True, editable=False, verbose_name='Longitude')),
                ('latitude', models.FloatField(blank=True, editable=False, verbose_name='Latitude')),
                ('position', django.contrib.gis.db.models.fields.PointField(blank=True, null=True, srid=4326, verbose_name='Position')),
                ('is_public', models.BooleanField(db_index=True, default=False, verbose_name='Is public?')),
                ('street', models.CharField(blank=True, max_length=250, verbose_name='Street')),
                ('postal_code', models.CharField(blank=True, max_length=25, verbose_name='Postal code')),
                ('city', models.CharField(blank=True, max_length=100, verbose_name='City')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('added_on', models.DateTimeField(default=datetime.datetime.now, editable=False)),
                ('updated_on', models.DateTimeField(editable=False)),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='places.Category')),
                ('site', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='sites.Site')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Place',
                'verbose_name_plural': 'Places',
            },
        ),
        migrations.AlterUniqueTogether(
            name='place',
            unique_together=set([('site', 'slug')]),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Place'
        db.create_table(u'places_place', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['sites.Site'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('longitude', self.gf('django.db.models.fields.FloatField')(blank=True)),
            ('latitude', self.gf('django.db.models.fields.FloatField')(blank=True)),
            ('position', self.gf('django.contrib.gis.db.models.fields.PointField')(null=True, blank=True)),
            ('is_public', self.gf('django.db.models.fields.BooleanField')(db_index=True)),
            ('street', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('postal_code', self.gf('django.db.models.fields.CharField')(max_length=25, blank=True)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('added_on', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('updated_on', self.gf('django.db.models.fields.DateTimeField')()),
        ))
        db.send_create_signal(u'places', ['Place'])

        # Adding unique constraint on 'Place', fields ['site', 'slug']
        db.create_unique(u'places_place', ['site_id', 'slug'])


    def backwards(self, orm):
        # Removing unique constraint on 'Place', fields ['site', 'slug']
        db.delete_unique(u'places_place', ['site_id', 'slug'])

        # Deleting model 'Place'
        db.delete_table(u'places_place')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'places.place': {
            'Meta': {'unique_together': "(('site', 'slug'),)", 'object_name': 'Place'},
            'added_on': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'db_index': 'True'}),
            'latitude': ('django.db.models.fields.FloatField', [], {'blank': 'True'}),
            'longitude': ('django.db.models.fields.FloatField', [], {'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'position': ('django.contrib.gis.db.models.fields.PointField', [], {'null': 'True', 'blank': 'True'}),
            'postal_code': ('django.db.models.fields.CharField', [], {'max_length': '25', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'street': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'updated_on': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'sites.site': {
            'Meta': {'ordering': "(u'domain',)", 'object_name': 'Site', 'db_table': "u'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['places']


# -*- coding: utf-8 -*-
"""Places app places_finder template tags."""
from __future__ import unicode_literals

from django import template
from django.conf import settings

from places.models import Place
from places.utils import sign_googleapi_url

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

register = template.Library()


@register.inclusion_tag('places/inclusion_tag/staticmap.html')
def staticmap(latitude, longitude, html_size="200x500", alt=None):
    """Return a static map."""
    try:
        height, width = html_size.split("x")
        height, width = int(height), int(width)
    except ValueError:
        err = 'staticmap third attribute must be in the form WIDTHxHEIGHT'
        err += '(like "300x400")'
        raise template.TemplateSyntaxError(err)
    key = getattr(settings, 'GOOGLE_API_KEY', None)
    secret = getattr(settings, 'GOOGLE_API_SECRET', None)
    params = {
        'center': '{latitude:f},{longitude:f}'.format(latitude=latitude, longitude=longitude),
        'zoom': "17",
        'maptype': "hybrid",
        'size': '{width}x{height}'.format(width=width, height=height),
        'sensor': 'false'
    }
    if key is not None:
        params['key'] = key
    url = "{path}?{query}".format(path="https://maps.googleapis.com/maps/api/staticmap",
                                  query=urlencode(params))
    if secret is not None:
        url = sign_googleapi_url(url, secret)

    return {
        'url': url,
        'lat': latitude,
        'long': longitude,
        'height': height,
        'width': width,
        'alt': alt,
    }


@register.simple_tag
def user_places_count(user):
    """Count the number of places for a given user."""
    return Place.objects.filter(user=user).count()
